@@ -63,6 +63,8 @@
 
 static HWND parent_wnd, g_edit;
 
+static HWND (WINAPI *pCreateUpDownControl)(DWORD, INT, INT, INT, INT,
+    HWND, INT, HINSTANCE, HWND, INT, INT, INT);
 static BOOL (WINAPI *pSetWindowSubclass)(HWND, SUBCLASSPROC, UINT_PTR, DWORD_PTR);
 
 static struct msg_sequence *sequences[NUM_MSG_SEQUENCES];
@@ -695,13 +697,13 @@ static void test_updown_base(void)
     r = SendMessageA(updown, UDM_SETPOS, 0, 10);
     expect(50, r);
 
-    GetWindowTextA(g_edit, text, sizeof(text)/sizeof(CHAR));
+    GetWindowTextA(g_edit, text, ARRAY_SIZE(text));
     ok(lstrcmpA(text, "10") == 0, "Expected '10', got '%s'\n", text);
 
     r = SendMessageA(updown, UDM_SETBASE, 16, 0);
     expect(10, r);
 
-    GetWindowTextA(g_edit, text, sizeof(text)/sizeof(CHAR));
+    GetWindowTextA(g_edit, text, ARRAY_SIZE(text));
     /* FIXME: currently hex output isn't properly formatted, but for this
        test only change from initial text matters */
     ok(lstrcmpA(text, "10") != 0, "Expected '0x000A', got '%s'\n", text);
@@ -835,20 +837,20 @@ static void test_UDS_SETBUDDYINT(void)
     style = GetWindowLongA(updown, GWL_STYLE);
     ok(style & UDS_SETBUDDYINT, "Expected UDS_SETBUDDY to be set\n");
     SendMessageA(updown, UDM_SETPOS, 0, 20);
-    GetWindowTextA(g_edit, text, sizeof(text)/sizeof(CHAR));
+    GetWindowTextA(g_edit, text, ARRAY_SIZE(text));
     ok(lstrlenA(text) == 0, "Expected empty string\n");
     DestroyWindow(updown);
 
     /* creating with UDS_SETBUDDYINT */
     updown = create_updown_control(UDS_SETBUDDYINT | UDS_ALIGNRIGHT, g_edit);
-    GetWindowTextA(g_edit, text, sizeof(text)/sizeof(CHAR));
+    GetWindowTextA(g_edit, text, ARRAY_SIZE(text));
     /* 50 is initial value here */
     ok(lstrcmpA(text, "50") == 0, "Expected '50', got '%s'\n", text);
     /* now remove style flag */
     style = GetWindowLongA(updown, GWL_STYLE);
     SetWindowLongA(updown, GWL_STYLE, style & ~UDS_SETBUDDYINT);
     SendMessageA(updown, UDM_SETPOS, 0, 20);
-    GetWindowTextA(g_edit, text, sizeof(text)/sizeof(CHAR));
+    GetWindowTextA(g_edit, text, ARRAY_SIZE(text));
     ok(lstrcmpA(text, "20") == 0, "Expected '20', got '%s'\n", text);
     /* set edit text directly, check position */
     strcpy(text, "10");
@@ -870,7 +872,7 @@ static void test_UDS_SETBUDDYINT(void)
     style = GetWindowLongA(updown, GWL_STYLE);
     SetWindowLongA(updown, GWL_STYLE, style | UDS_SETBUDDYINT);
     SendMessageA(updown, UDM_SETPOS, 0, 30);
-    GetWindowTextA(g_edit, text, sizeof(text)/sizeof(CHAR));
+    GetWindowTextA(g_edit, text, ARRAY_SIZE(text));
     ok(lstrcmpA(text, "30") == 0, "Expected '30', got '%s'\n", text);
     DestroyWindow(updown);
 }
@@ -882,7 +884,7 @@ static void test_CreateUpDownControl(void)
     RECT rect;
 
     GetClientRect(parent_wnd, &rect);
-    updown = CreateUpDownControl(WS_CHILD | WS_BORDER | WS_VISIBLE,
+    updown = pCreateUpDownControl(WS_CHILD | WS_BORDER | WS_VISIBLE,
         0, 0, rect.right, rect.bottom, parent_wnd, 1, GetModuleHandleA(NULL), g_edit, 100, 10, 50);
     ok(updown != NULL, "Failed to create control.\n");
 
@@ -898,13 +900,22 @@ static void test_CreateUpDownControl(void)
     DestroyWindow(updown);
 }
 
+static void init_functions(void)
+{
+    HMODULE hComCtl32 = LoadLibraryA("comctl32.dll");
+
+#define X(f) p##f = (void*)GetProcAddress(hComCtl32, #f);
+#define X2(f, ord) p##f = (void*)GetProcAddress(hComCtl32, (const char *)ord);
+    X(CreateUpDownControl);
+    X2(SetWindowSubclass, 410);
+#undef X
+#undef X2
+}
+
 START_TEST(updown)
 {
-    HMODULE mod = GetModuleHandleA("comctl32.dll");
+    init_functions();
 
-    pSetWindowSubclass = (void*)GetProcAddress(mod, (LPSTR)410);
-
-    InitCommonControls();
     init_msg_sequences(sequences, NUM_MSG_SEQUENCES);
 
     parent_wnd = create_parent_window();
