@@ -557,7 +557,7 @@ HRESULT dxbc_add_section(struct dxbc *dxbc, DWORD tag, const char *data, DWORD d
     return S_OK;
 }
 
-HRESULT dxbc_init(struct dxbc *dxbc, UINT size)
+HRESULT dxbc_init(struct dxbc *dxbc, unsigned int size)
 {
     TRACE("dxbc %p, size %u.\n", dxbc, size);
 
@@ -2138,9 +2138,12 @@ static void debug_dump_ir_constructor(const struct hlsl_ir_constructor *construc
 
 static const char *debug_writemask(DWORD writemask)
 {
-    char string[5], components[] = {'x', 'y', 'z', 'w'};
+    static const char components[] = {'x', 'y', 'z', 'w'};
+    char string[5];
     unsigned int i = 0, pos = 0;
 
+    assert(!(writemask & ~BWRITERSP_WRITEMASK_ALL));
+
     while (writemask)
     {
         if (writemask & 1)
