@@ -1096,7 +1096,7 @@ static void macho_finish_stabs(struct module* module, struct hash_table* ht_symt
 
         if (ste->is_public && !(dbghelp_options & SYMOPT_NO_PUBLICS))
         {
-            symt_new_public(module, ste->compiland, ste->ht_elt.name, ste->addr, 0);
+            symt_new_public(module, ste->compiland, ste->ht_elt.name, ste->is_code, ste->addr, 0);
         }
     }
 }
