@@ -1309,6 +1309,13 @@ void opentype_get_font_metrics(struct file_stream_desc *stream_desc, DWRITE_FONT
             metrics->hasTypographicMetrics = TRUE;
         }
     }
+    else {
+        metrics->strikethroughPosition = metrics->designUnitsPerEm / 3;
+        if (tt_hhea) {
+            metrics->ascent = GET_BE_WORD(tt_hhea->ascender);
+            metrics->descent = abs((SHORT)GET_BE_WORD(tt_hhea->descender));
+        }
+    }
 
     if (tt_post) {
         metrics->underlinePosition = GET_BE_WORD(tt_post->underlinePosition);
@@ -1553,7 +1560,7 @@ static BOOL opentype_decode_namerecord(const TT_NAME_V0 *header, BYTE *storage_a
         UINT codepage;
 
         codepage = get_name_record_codepage(platform, encoding);
-        get_name_record_locale(platform, lang_id, locale, sizeof(locale)/sizeof(WCHAR));
+        get_name_record_locale(platform, lang_id, locale, ARRAY_SIZE(locale));
 
         if (codepage) {
             DWORD len = MultiByteToWideChar(codepage, 0, (LPSTR)(storage_area + offset), length, NULL, 0);
@@ -1730,14 +1737,25 @@ HRESULT opentype_get_font_facename(struct file_stream_desc *stream_desc, WCHAR *
         BOOL exists;
 
         exists = FALSE;
-        if (GetSystemDefaultLocaleName(localeW, sizeof(localeW)/sizeof(WCHAR)))
+        if (GetSystemDefaultLocaleName(localeW, ARRAY_SIZE(localeW)))
             IDWriteLocalizedStrings_FindLocaleName(lfnames, localeW, &index, &exists);
 
         if (!exists)
             IDWriteLocalizedStrings_FindLocaleName(lfnames, enusW, &index, &exists);
 
-        if (exists)
-            IDWriteLocalizedStrings_GetString(lfnames, index, lfname, LF_FACESIZE);
+        if (exists) {
+            UINT32 length = 0;
+            WCHAR *nameW;
+
+            IDWriteLocalizedStrings_GetStringLength(lfnames, index, &length);
+            nameW = heap_alloc((length + 1) * sizeof(WCHAR));
+            if (nameW) {
+                *nameW = 0;
+                IDWriteLocalizedStrings_GetString(lfnames, index, nameW, length + 1);
+                lstrcpynW(lfname, nameW, LF_FACESIZE);
+                heap_free(nameW);
+            }
+        }
 
         IDWriteLocalizedStrings_Release(lfnames);
     }
@@ -1801,7 +1819,7 @@ HRESULT opentype_get_typographic_features(IDWriteFontFace *fontface, UINT32 scri
     UINT8 i;
 
     *count = 0;
-    for (i = 0; i < sizeof(tables)/sizeof(tables[0]); i++) {
+    for (i = 0; i < ARRAY_SIZE(tables); i++) {
         const OT_ScriptList *scriptlist;
         const GPOS_GSUB_Header *header;
         const OT_Script *script;
