@@ -1594,7 +1594,7 @@ static BOOL pathdrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags, const REC
         void *outline;
 
         dwSize = GetGlyphOutlineW(dev->hdc, str[idx], ggo_flags, &gm, 0, NULL, &identity);
-        if (dwSize == GDI_ERROR) return FALSE;
+        if (dwSize == GDI_ERROR) continue;
 
         /* add outline only if char is printable */
         if(dwSize)
@@ -2248,5 +2248,6 @@ const struct gdi_dc_funcs path_driver =
     NULL,                               /* pUnrealizePalette */
     NULL,                               /* pWidenPath */
     NULL,                               /* wine_get_wgl_driver */
+    NULL,                               /* wine_get_vulkan_driver */
     GDI_PRIORITY_PATH_DRV               /* priority */
 };
