@@ -772,7 +772,7 @@ static BOOL PROFILE_Open( LPCWSTR filename, BOOL write_access )
     else
     {
         LPWSTR dummy;
-        GetFullPathNameW(filename, sizeof(buffer)/sizeof(buffer[0]), buffer, &dummy);
+        GetFullPathNameW(filename, ARRAY_SIZE(buffer), buffer, &dummy);
     }
         
     TRACE("path: %s\n", debugstr_w(buffer));
@@ -879,7 +879,7 @@ static INT PROFILE_GetSection( PROFILESECTION *section, LPCWSTR section_name,
             for (key = section->key; key; key = key->next)
             {
                 if (len <= 2) break;
-                if (!*key->name) continue;  /* Skip empty lines */
+                if (!*key->name && !key->value) continue;  /* Skip empty lines */
                 if (IS_ENTRY_COMMENT(key->name)) continue;  /* Skip comments */
                 if (!return_values && !key->value) continue;  /* Skip lines w.o. '=' */
                 PROFILE_CopyEntry( buffer, key->name, len - 1, 0 );
@@ -1238,9 +1238,8 @@ UINT WINAPI GetPrivateProfileIntW( LPCWSTR section, LPCWSTR entry,
     UNICODE_STRING bufferW;
     ULONG result;
 
-    if (GetPrivateProfileStringW( section, entry, emptystringW,
-                                   buffer, sizeof(buffer)/sizeof(WCHAR),
-                                   filename ) == 0)
+    if (GetPrivateProfileStringW( section, entry, emptystringW, buffer, ARRAY_SIZE( buffer ),
+                                  filename ) == 0)
         return def_val;
 
     /* FIXME: if entry can be found but it's empty, then Win16 is
@@ -1391,7 +1390,7 @@ BOOL WINAPI WritePrivateProfileStringW( LPCWSTR section, LPCWSTR entry,
             SetLastError(ERROR_FILE_NOT_FOUND);
         } else {
             ret = PROFILE_SetString( section, entry, string, FALSE);
-            PROFILE_FlushFile();
+            if (ret) ret = PROFILE_FlushFile();
         }
     }
 
@@ -1447,11 +1446,10 @@ BOOL WINAPI WritePrivateProfileSectionW( LPCWSTR section,
     else if (PROFILE_Open( filename, TRUE )) {
         if (!string) {/* delete the named section*/
 	    ret = PROFILE_SetString(section,NULL,NULL, FALSE);
-	    PROFILE_FlushFile();
         } else {
 	    PROFILE_DeleteAllKeys(section);
 	    ret = TRUE;
-	    while(*string) {
+	    while(*string && ret) {
                 LPWSTR buf = HeapAlloc( GetProcessHeap(), 0, (strlenW(string)+1) * sizeof(WCHAR) );
                 strcpyW( buf, string );
                 if((p = strchrW( buf, '='))) {
@@ -1461,8 +1459,8 @@ BOOL WINAPI WritePrivateProfileSectionW( LPCWSTR section,
                 HeapFree( GetProcessHeap(), 0, buf );
                 string += strlenW(string)+1;
             }
-            PROFILE_FlushFile();
         }
+        if (ret) ret = PROFILE_FlushFile();
     }
 
     RtlLeaveCriticalSection( &PROFILE_CritSect );
@@ -1744,7 +1742,7 @@ BOOL WINAPI WritePrivateProfileStructW (LPCWSTR section, LPCWSTR key,
 
     if (PROFILE_Open( filename, TRUE )) {
         ret = PROFILE_SetString( section, key, outstring, FALSE);
-        PROFILE_FlushFile();
+        if (ret) ret = PROFILE_FlushFile();
     }
 
     RtlLeaveCriticalSection( &PROFILE_CritSect );
