@@ -25,6 +25,7 @@
 typedef unsigned char MSVCP_bool;
 typedef SIZE_T MSVCP_size_t;
 
+#define MSVCP_SIZE_T_MAX (~(MSVCP_size_t)0)
 #if _MSVCP_VER >= 100
 typedef __int64 DECLSPEC_ALIGN(8) streamoff;
 typedef __int64 DECLSPEC_ALIGN(8) streamsize;
@@ -47,6 +48,10 @@ extern void* (__cdecl *MSVCRT_operator_new)(MSVCP_size_t);
 extern void (__cdecl *MSVCRT_operator_delete)(void*);
 extern void* (__cdecl *MSVCRT_set_new_handler)(void*);
 
+#if _MSVCP_VER >= 140
+extern int* (__cdecl *UCRTBASE___processing_throw)(void);
+#endif
+
 #if _MSVCP_VER >= 110
 /* keep in sync with msvcrt/lock.c */
 typedef struct cs_queue
