@@ -1016,6 +1016,15 @@ int CDECL MSVCRT__vscprintf( const char *format, __ms_va_list valist )
     return MSVCRT_vsnprintf( NULL, INT_MAX, format, valist );
 }
 
+/*********************************************************************
+ *              _vscprintf_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__vscprintf_l(const char *format,
+        MSVCRT__locale_t locale, __ms_va_list valist)
+{
+    return MSVCRT_vsnprintf_l(NULL, INT_MAX, format, locale, valist);
+}
+
 /*********************************************************************
  *		_vscprintf_p_l (MSVCRT.@)
  */
@@ -1806,6 +1815,14 @@ INT CDECL MSVCRT_iswalnum( MSVCRT_wchar_t wc )
     return isalnumW( wc );
 }
 
+/*********************************************************************
+ *		_iswalnum_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__iswalnum_l( MSVCRT_wchar_t wc, MSVCRT__locale_t locale )
+{
+    return isalnumW( wc );
+}
+
 /*********************************************************************
  *		iswalpha (MSVCRT.@)
  */
@@ -1830,6 +1847,14 @@ INT CDECL MSVCRT_iswcntrl( MSVCRT_wchar_t wc )
     return iscntrlW( wc );
 }
 
+/*********************************************************************
+ *		_iswcntrl_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__iswcntrl_l( MSVCRT_wchar_t wc, MSVCRT__locale_t locale )
+{
+    return iscntrlW( wc );
+}
+
 /*********************************************************************
  *		iswdigit (MSVCRT.@)
  */
@@ -1854,6 +1879,14 @@ INT CDECL MSVCRT_iswgraph( MSVCRT_wchar_t wc )
     return isgraphW( wc );
 }
 
+/*********************************************************************
+ *		_iswgraph_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__iswgraph_l( MSVCRT_wchar_t wc, MSVCRT__locale_t locale )
+{
+    return isgraphW( wc );
+}
+
 /*********************************************************************
  *		iswlower (MSVCRT.@)
  */
@@ -1862,6 +1895,14 @@ INT CDECL MSVCRT_iswlower( MSVCRT_wchar_t wc )
     return islowerW( wc );
 }
 
+/*********************************************************************
+ *		_iswlower_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__iswlower_l( MSVCRT_wchar_t wc, MSVCRT__locale_t locale )
+{
+    return islowerW( wc );
+}
+
 /*********************************************************************
  *		iswprint (MSVCRT.@)
  */
@@ -1870,6 +1911,14 @@ INT CDECL MSVCRT_iswprint( MSVCRT_wchar_t wc )
     return isprintW( wc );
 }
 
+/*********************************************************************
+ *		_iswprint_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__iswprint_l( MSVCRT_wchar_t wc, MSVCRT__locale_t locale )
+{
+    return isprintW( wc );
+}
+
 /*********************************************************************
  *		_iswpunct_l (MSVCRT.@)
  */
@@ -1910,6 +1959,14 @@ INT CDECL MSVCRT_iswupper( MSVCRT_wchar_t wc )
     return isupperW( wc );
 }
 
+/*********************************************************************
+ *		_iswupper_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__iswupper_l( MSVCRT_wchar_t wc, MSVCRT__locale_t locale )
+{
+    return isupperW( wc );
+}
+
 /*********************************************************************
  *		iswxdigit (MSVCRT.@)
  */
@@ -1918,6 +1975,14 @@ INT CDECL MSVCRT_iswxdigit( MSVCRT_wchar_t wc )
     return isxdigitW( wc );
 }
 
+/*********************************************************************
+ *		_iswxdigit_l (MSVCRT.@)
+ */
+int CDECL MSVCRT__iswxdigit_l( MSVCRT_wchar_t wc, MSVCRT__locale_t locale )
+{
+    return isxdigitW( wc );
+}
+
 /*********************************************************************
  *		_iswctype_l (MSVCRT.@)
  */
