@@ -745,7 +745,7 @@ DECLSPEC_HIDDEN void * WINAPI relay_trace_entry( struct relay_descr *descr, unsi
             TRACE( "%08lx", stack[i] );
             break;
         }
-        if (!is_ret_val( arg_types[i] )) TRACE( "," );
+        if (!is_ret_val( arg_types[i+1] )) TRACE( "," );
     }
     *nb_args = i;
     TRACE( ") ret=%08lx\n", stack[-1] );
@@ -1197,10 +1197,10 @@ void WINAPI DECLSPEC_HIDDEN __regs_SNOOP_Entry( void **stack )
 
 
 	while (*rets) {
-		for (i=0;i<sizeof((*rets)->entry)/sizeof((*rets)->entry[0]);i++)
+		for (i=0;i<ARRAY_SIZE( (*rets)->entry );i++)
 			if (!(*rets)->entry[i].origreturn)
 				break;
-		if (i!=sizeof((*rets)->entry)/sizeof((*rets)->entry[0]))
+		if (i!=ARRAY_SIZE( (*rets)->entry ))
 			break;
 		rets = &((*rets)->next);
 	}
