@@ -804,7 +804,7 @@ static const IBaseFilterVtbl MPEGSplitter_Vtbl =
     Parser_SetSyncSource,
     Parser_GetSyncSource,
     Parser_EnumPins,
-    Parser_FindPin,
+    BaseFilterImpl_FindPin,
     Parser_QueryFilterInfo,
     Parser_JoinFilterGraph,
     Parser_QueryVendorInfo
