@@ -99,7 +99,7 @@ static void filetime_add_ms(FILETIME *ft, LONGLONG ms)
         LONGLONG ll;
     } *ftll = (union u_ftll *)ft;
 
-    ftll->ll += ms * (ULONGLONG)10000;
+    ftll->ll += ms * (LONGLONG)10000;
 }
 
 static void filetime_add_minutes(FILETIME *ft, LONG minutes)
@@ -117,7 +117,7 @@ static void filetime_add_days(FILETIME *ft, LONG days)
     filetime_add_hours(ft, (LONGLONG)days * 24);
 }
 
-static void filetime_add_weeks(FILETIME *ft, ULONG weeks)
+static void filetime_add_weeks(FILETIME *ft, LONG weeks)
 {
     filetime_add_days(ft, (LONGLONG)weeks * 7);
 }
@@ -170,6 +170,9 @@ static BOOL trigger_get_next_runtime(const TASK_TRIGGER *trigger, const FILETIME
     FileTimeToSystemTime(current_ft, &current_st);
 
     get_begin_time(trigger, &begin_ft);
+    if (CompareFileTime(&begin_ft, current_ft) < 0)
+        begin_ft = *current_ft;
+
     get_end_time(trigger, &end_ft);
 
     switch (trigger->TriggerType)
@@ -194,6 +197,9 @@ static BOOL trigger_get_next_runtime(const TASK_TRIGGER *trigger, const FILETIME
         break;
 
     case TASK_TIME_TRIGGER_DAILY:
+        if (!trigger->Type.Daily.DaysInterval)
+            break; /* avoid infinite loop */
+
         st = current_st;
         st.wHour = trigger->wStartHour;
         st.wMinute = trigger->wStartMinute;
@@ -250,7 +256,7 @@ static BOOL trigger_get_next_runtime(const TASK_TRIGGER *trigger, const FILETIME
     return FALSE;
 }
 
-static BOOL job_get_next_runtime(struct job_t *job, FILETIME *current_ft, FILETIME *next_rt)
+static BOOL job_get_next_runtime(struct job_t *job, const FILETIME *current_ft, FILETIME *next_rt)
 {
     FILETIME trigger_rt;
     BOOL have_next_rt = FALSE;
@@ -307,7 +313,7 @@ BOOL get_next_runtime(LARGE_INTEGER *rt)
     return have_next_rt;
 }
 
-static BOOL job_runs_at(struct job_t *job, FILETIME *begin_ft, FILETIME *end_ft)
+static BOOL job_runs_at(struct job_t *job, const FILETIME *begin_ft, const FILETIME *end_ft)
 {
     FILETIME job_ft;
 
