@@ -87,8 +87,6 @@ static const shvheader mycomputer_header[] =
     { NULL, 0, IDS_SHV_COLUMN7, SHCOLSTATE_TYPE_STR | SHCOLSTATE_ONBYDEFAULT, LVCFMT_RIGHT, 10 },
 };
 
-#define MYCOMPUTERSHELLVIEWCOLUMNS sizeof(mycomputer_header)/sizeof(shvheader)
-
 /**************************************************************************
 *    ISF_MyComputer_Constructor
 */
@@ -327,7 +325,7 @@ static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
                     DWORD size;
                     LONG r;
 
-                    size = sizeof(iid) / sizeof(iid[0]);
+                    size = ARRAY_SIZE(iid);
                     r = RegEnumKeyExW(hkey, i, iid, &size, 0, NULL, NULL, NULL);
                     if (ERROR_SUCCESS == r)
                     {
@@ -700,8 +698,8 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
                 static const WCHAR wszCloseBracket[] = {')',0};
                 WCHAR wszDrive[32 /* label */ + 6 /* ' (C:)'\0 */] = {0};
 
-                GetVolumeInformationW (pszPath, wszDrive, sizeof(wszDrive)/sizeof(wszDrive[0]) - 5,
-                        NULL, NULL, NULL, NULL, 0);
+                GetVolumeInformationW (pszPath, wszDrive, ARRAY_SIZE(wszDrive) - 5, NULL, NULL,
+                        NULL, NULL, 0);
                 strcatW (wszDrive, wszOpenBracket);
                 lstrcpynW (wszDrive + strlenW(wszDrive), pszPath, 3);
                 strcatW (wszDrive, wszCloseBracket);
@@ -799,7 +797,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDefaultColumnState (
 
     TRACE ("(%p)->(%d %p)\n", This, iColumn, pcsFlags);
 
-    if (!pcsFlags || iColumn >= MYCOMPUTERSHELLVIEWCOLUMNS)
+    if (!pcsFlags || iColumn >= ARRAY_SIZE(mycomputer_header))
         return E_INVALIDARG;
 
     *pcsFlags = mycomputer_header[iColumn].pcsFlags;
@@ -826,7 +824,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDetailsOf (IShellFolder2 *iface,
 
     TRACE ("(%p)->(%p %i %p)\n", This, pidl, iColumn, psd);
 
-    if (!psd || iColumn >= MYCOMPUTERSHELLVIEWCOLUMNS)
+    if (!psd || iColumn >= ARRAY_SIZE(mycomputer_header))
         return E_INVALIDARG;
 
     if (!pidl)
@@ -871,7 +869,7 @@ static HRESULT WINAPI ISF_MyComputer_fnMapColumnToSCID (IShellFolder2 *iface, UI
 
     TRACE("(%p)->(%u %p)\n", This, column, scid);
 
-    if (column >= MYCOMPUTERSHELLVIEWCOLUMNS)
+    if (column >= ARRAY_SIZE(mycomputer_header))
         return E_INVALIDARG;
 
     return shellfolder_map_column_to_scid(mycomputer_header, column, scid);
