@@ -288,12 +288,13 @@ static BOOL SHELL_ConfirmDialogW(HWND hWnd, int nKindOfDialog, LPCWSTR szDir, FI
 
         if (!SHELL_ConfirmIDs(nKindOfDialog, &ids)) return FALSE;
 
-	LoadStringW(shell32_hInstance, ids.caption_resource_id, szCaption, sizeof(szCaption)/sizeof(WCHAR));
-	LoadStringW(shell32_hInstance, ids.text_resource_id, szText, sizeof(szText)/sizeof(WCHAR));
+        LoadStringW(shell32_hInstance, ids.caption_resource_id, szCaption, ARRAY_SIZE(szCaption));
+        LoadStringW(shell32_hInstance, ids.text_resource_id, szText, ARRAY_SIZE(szText));
+
+        args[0] = (DWORD_PTR)szDir;
+        FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY,
+            szText, 0, 0, szBuffer, ARRAY_SIZE(szBuffer), (__ms_va_list*)args);
 
-	args[0] = (DWORD_PTR)szDir;
-	FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY,
-	               szText, 0, 0, szBuffer, sizeof(szBuffer)/sizeof(szBuffer[0]), (__ms_va_list*)args);
         hIcon = LoadIconW(ids.hIconInstance, (LPWSTR)MAKEINTRESOURCE(ids.icon_resource_id));
 
         ret = SHELL_ConfirmMsgBox(hWnd, szBuffer, szCaption, hIcon, op && op->bManyItems);
@@ -322,7 +323,7 @@ static DWORD SHELL32_AnsiToUnicodeBuf(LPCSTR aPath, LPWSTR *wPath, DWORD minChar
 	if (len < minChars)
 	  len = minChars;
 
-	*wPath = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+	*wPath = heap_alloc(len * sizeof(WCHAR));
 	if (*wPath)
 	{
 	  MultiByteToWideChar(CP_ACP, 0, aPath, -1, *wPath, len);
@@ -331,11 +332,6 @@ static DWORD SHELL32_AnsiToUnicodeBuf(LPCSTR aPath, LPWSTR *wPath, DWORD minChar
 	return E_OUTOFMEMORY;
 }
 
-static void SHELL32_FreeUnicodeBuf(LPWSTR wPath)
-{
-	HeapFree(GetProcessHeap(), 0, wPath);
-}
-
 HRESULT WINAPI SHIsFileAvailableOffline(LPCWSTR path, LPDWORD status)
 {
     FIXME("(%s, %p) stub\n", debugstr_w(path), status);
@@ -406,7 +402,7 @@ static DWORD SHNotifyCreateDirectoryA(LPCSTR path, LPSECURITY_ATTRIBUTES sec)
 	if (!retCode)
 	{
 	  retCode = SHNotifyCreateDirectoryW(wPath, sec);
-	  SHELL32_FreeUnicodeBuf(wPath);
+	  heap_free(wPath);
 	}
 	return retCode;
 }
@@ -460,7 +456,7 @@ static DWORD SHNotifyRemoveDirectoryA(LPCSTR path)
 	if (!retCode)
 	{
 	  retCode = SHNotifyRemoveDirectoryW(wPath);
-	  SHELL32_FreeUnicodeBuf(wPath);
+	  heap_free(wPath);
 	}
 	return retCode;
 }
@@ -524,7 +520,7 @@ static DWORD SHNotifyDeleteFileA(LPCSTR path)
 	if (!retCode)
 	{
 	  retCode = SHNotifyDeleteFileW(wPath);
-	  SHELL32_FreeUnicodeBuf(wPath);
+	  heap_free(wPath);
 	}
 	return retCode;
 }
@@ -720,7 +716,7 @@ int WINAPI SHCreateDirectoryExA(HWND hWnd, LPCSTR path, LPSECURITY_ATTRIBUTES se
 	if (!retCode)
 	{
 	  retCode = SHCreateDirectoryExW(hWnd, wPath, sec);
-	  SHELL32_FreeUnicodeBuf(wPath);
+	  heap_free(wPath);
 	}
 	return retCode;
 }
@@ -886,12 +882,16 @@ int WINAPI SHFileOperationA(LPSHFILEOPSTRUCTA lpFileOp)
 	  if (ForFree)
 	  {
 	    retCode = SHFileOperationW(&nFileOp);
-	    HeapFree(GetProcessHeap(), 0, ForFree); /* we cannot use wString, it was changed */
+	    /* Windows 95/98 returns S_OK for this case. */
+	    if (retCode == ERROR_ACCESS_DENIED && (GetVersion() & 0x80000000))
+	      retCode = S_OK;
+
+	    heap_free(ForFree); /* we cannot use wString, it was changed */
 	    break;
 	  }
 	  else
 	  {
-	    wString = ForFree = HeapAlloc(GetProcessHeap(), 0, size * sizeof(WCHAR));
+	    wString = ForFree = heap_alloc(size * sizeof(WCHAR));
 	    if (ForFree) continue;
 	    retCode = ERROR_OUTOFMEMORY;
 	    nFileOp.fAnyOperationsAborted = TRUE;
@@ -943,18 +943,18 @@ static void add_file_to_entry(FILE_ENTRY *feFile, LPCWSTR szFile)
     DWORD dwLen = lstrlenW(szFile) + 1;
     LPCWSTR ptr;
 
-    feFile->szFullPath = HeapAlloc(GetProcessHeap(), 0, dwLen * sizeof(WCHAR));
+    feFile->szFullPath = heap_alloc(dwLen * sizeof(WCHAR));
     lstrcpyW(feFile->szFullPath, szFile);
 
     ptr = StrRChrW(szFile, NULL, '\\');
     if (ptr)
     {
         dwLen = ptr - szFile + 1;
-        feFile->szDirectory = HeapAlloc(GetProcessHeap(), 0, dwLen * sizeof(WCHAR));
+        feFile->szDirectory = heap_alloc(dwLen * sizeof(WCHAR));
         lstrcpynW(feFile->szDirectory, szFile, dwLen);
 
         dwLen = lstrlenW(feFile->szFullPath) - dwLen + 1;
-        feFile->szFilename = HeapAlloc(GetProcessHeap(), 0, dwLen * sizeof(WCHAR));
+        feFile->szFilename = heap_alloc(dwLen * sizeof(WCHAR));
         lstrcpyW(feFile->szFilename, ptr + 1); /* skip over backslash */
     }
     feFile->bFromWildcard = FALSE;
@@ -970,7 +970,7 @@ static LPWSTR wildcard_to_file(LPCWSTR szWildCard, LPCWSTR szFileName)
     dwDirLen = ptr - szWildCard + 1;
 
     dwFullLen = dwDirLen + lstrlenW(szFileName) + 1;
-    szFullPath = HeapAlloc(GetProcessHeap(), 0, dwFullLen * sizeof(WCHAR));
+    szFullPath = heap_alloc(dwFullLen * sizeof(WCHAR));
 
     lstrcpynW(szFullPath, szWildCard, dwDirLen + 1);
     lstrcatW(szFullPath, szFileName);
@@ -998,7 +998,7 @@ static void parse_wildcard_files(FILE_LIST *flList, LPCWSTR szFile, LPDWORD pdwL
         file->bFromWildcard = TRUE;
         file->attributes = wfd.dwFileAttributes;
         if (IsAttribDir(file->attributes)) flList->bAnyDirectories = TRUE;
-        HeapFree(GetProcessHeap(), 0, szFullPath);
+        heap_free(szFullPath);
     }
 
     FindClose(hFile);
@@ -1009,6 +1009,7 @@ static HRESULT parse_file_list(FILE_LIST *flList, LPCWSTR szFiles)
 {
     LPCWSTR ptr = szFiles;
     WCHAR szCurFile[MAX_PATH];
+    WCHAR *p;
     DWORD i = 0;
 
     if (!szFiles)
@@ -1024,8 +1025,7 @@ static HRESULT parse_file_list(FILE_LIST *flList, LPCWSTR szFiles)
     if (!szFiles[0])
         return ERROR_ACCESS_DENIED;
         
-    flList->feFiles = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-                                flList->num_alloc * sizeof(FILE_ENTRY));
+    flList->feFiles = heap_alloc_zero(flList->num_alloc * sizeof(FILE_ENTRY));
 
     while (*ptr)
     {
@@ -1044,6 +1044,8 @@ static HRESULT parse_file_list(FILE_LIST *flList, LPCWSTR szFiles)
             flList->feFiles[i].bFromRelative = FALSE;
         }
 
+        for (p = szCurFile; *p; p++) if (*p == '/') *p = '\\';
+
         /* parse wildcard files if they are in the filename */
         if (StrPBrkW(szCurFile, wWildcardChars))
         {
@@ -1080,12 +1082,12 @@ static void destroy_file_list(FILE_LIST *flList)
 
     for (i = 0; i < flList->dwNumFiles; i++)
     {
-        HeapFree(GetProcessHeap(), 0, flList->feFiles[i].szDirectory);
-        HeapFree(GetProcessHeap(), 0, flList->feFiles[i].szFilename);
-        HeapFree(GetProcessHeap(), 0, flList->feFiles[i].szFullPath);
+        heap_free(flList->feFiles[i].szDirectory);
+        heap_free(flList->feFiles[i].szFilename);
+        heap_free(flList->feFiles[i].szFullPath);
     }
 
-    HeapFree(GetProcessHeap(), 0, flList->feFiles);
+    heap_free(flList->feFiles);
 }
 
 static void copy_dir_to_dir(FILE_OPERATION *op, const FILE_ENTRY *feFrom, LPCWSTR szDestPath)
@@ -1216,9 +1218,9 @@ static int copy_files(FILE_OPERATION *op, const FILE_LIST *flFrom, FILE_LIST *fl
             /* Free all but the first entry. */
             for (i = 1; i < flTo->dwNumFiles; i++)
             {
-                HeapFree(GetProcessHeap(), 0, flTo->feFiles[i].szDirectory);
-                HeapFree(GetProcessHeap(), 0, flTo->feFiles[i].szFilename);
-                HeapFree(GetProcessHeap(), 0, flTo->feFiles[i].szFullPath);
+                heap_free(flTo->feFiles[i].szDirectory);
+                heap_free(flTo->feFiles[i].szFilename);
+                heap_free(flTo->feFiles[i].szFullPath);
             }
 
             flTo->dwNumFiles = 1;
@@ -1312,7 +1314,7 @@ static BOOL confirm_delete_list(HWND hWnd, DWORD fFlags, BOOL fTrash, const FILE
         static const WCHAR format[] = {'%','d',0};
         WCHAR tmp[8];
 
-        wnsprintfW(tmp, sizeof(tmp)/sizeof(tmp[0]), format, flFrom->dwNumFiles);
+        wnsprintfW(tmp, ARRAY_SIZE(tmp), format, flFrom->dwNumFiles);
         return SHELL_ConfirmDialogW(hWnd, (fTrash?ASK_TRASH_MULTIPLE_ITEM:ASK_DELETE_MULTIPLE_ITEM), tmp, NULL);
     }
     else
@@ -1758,7 +1760,7 @@ HRESULT WINAPI SHPathPrepareForWriteW(HWND hwnd, IUnknown *modless, LPCWSTR path
             len = 1;
         else
             len = last_slash - path + 1;
-        temppath = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+        temppath = heap_alloc(len * sizeof(WCHAR));
         if (!temppath)
             return E_OUTOFMEMORY;
         StrCpyNW(temppath, path, len);
@@ -1781,7 +1783,7 @@ HRESULT WINAPI SHPathPrepareForWriteW(HWND hwnd, IUnknown *modless, LPCWSTR path
     /* check if we can access the directory */
     res = GetFileAttributesW(realpath);
 
-    HeapFree(GetProcessHeap(), 0, temppath);
+    heap_free(temppath);
 
     if (res == INVALID_FILE_ATTRIBUTES)
     {
