@@ -228,6 +228,7 @@ WNDPROC16 WINPROC_GetProc16( WNDPROC proc, BOOL unicode )
 static LRESULT call_window_proc16( HWND16 hwnd, UINT16 msg, WPARAM16 wParam, LPARAM lParam,
                                    LRESULT *result, void *arg )
 {
+    DPI_AWARENESS_CONTEXT awareness;
     WNDPROC16 func = arg;
     int index = winproc_to_index( func );
     CONTEXT context;
@@ -281,6 +282,7 @@ static LRESULT call_window_proc16( HWND16 hwnd, UINT16 msg, WPARAM16 wParam, LPA
         }
     }
 
+    awareness = SetThreadDpiAwarenessContext( GetWindowDpiAwarenessContext( HWND_32(hwnd) ));
     args.params[4] = hwnd;
     args.params[3] = msg;
     args.params[2] = wParam;
@@ -288,6 +290,7 @@ static LRESULT call_window_proc16( HWND16 hwnd, UINT16 msg, WPARAM16 wParam, LPA
     args.params[0] = LOWORD(lParam);
     WOWCallback16Ex( 0, WCB16_REGS, sizeof(args.params) + size, &args, (DWORD *)&context );
     *result = MAKELONG( LOWORD(context.Eax), LOWORD(context.Edx) );
+    SetThreadDpiAwarenessContext( awareness );
     return *result;
 }
 
