@@ -32,6 +32,7 @@
 #include "uxtheme.h"
 
 #include "wine/debug.h"
+#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(uxtheme);
 
@@ -48,7 +49,7 @@ static void free_paintbuffer(struct paintbuffer *buffer)
 {
     DeleteObject(buffer->bitmap);
     DeleteDC(buffer->memorydc);
-    HeapFree(GetProcessHeap(), 0, buffer);
+    heap_free(buffer);
 }
 
 static struct paintbuffer *get_buffer_obj(HPAINTBUFFER handle)
@@ -98,7 +99,7 @@ HPAINTBUFFER WINAPI BeginBufferedPaint(HDC targetdc, const RECT *rect,
     if (params)
         FIXME("painting parameters are ignored\n");
 
-    buffer = HeapAlloc(GetProcessHeap(), 0, sizeof(*buffer));
+    buffer = heap_alloc(sizeof(*buffer));
     buffer->targetdc = targetdc;
     buffer->rect = *rect;
     buffer->memorydc = CreateCompatibleDC(targetdc);
