@@ -24,6 +24,7 @@
 #include "webservices.h"
 
 #include "wine/debug.h"
+#include "wine/heap.h"
 #include "wine/list.h"
 #include "webservices_private.h"
 
@@ -41,14 +42,14 @@ struct error
     ULONG            magic;
     CRITICAL_SECTION cs;
     ULONG            prop_count;
-    struct prop      prop[sizeof(error_props)/sizeof(error_props[0])];
+    struct prop      prop[ARRAY_SIZE( error_props )];
 };
 
 #define ERROR_MAGIC (('E' << 24) | ('R' << 16) | ('R' << 8) | 'O')
 
 static struct error *alloc_error(void)
 {
-    static const ULONG count = sizeof(error_props)/sizeof(error_props[0]);
+    static const ULONG count = ARRAY_SIZE( error_props );
     struct error *ret;
     ULONG size = sizeof(*ret) + prop_size( error_props, count );
 
@@ -56,7 +57,9 @@ static struct error *alloc_error(void)
 
     ret->magic      = ERROR_MAGIC;
     InitializeCriticalSection( &ret->cs );
+#ifndef __MINGW32__
     ret->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": error.cs");
+#endif
 
     prop_init( error_props, count, ret->prop, &ret[1] );
     ret->prop_count = count;
@@ -65,7 +68,9 @@ static struct error *alloc_error(void)
 
 static void free_error( struct error *error )
 {
+#ifndef __MINGW32__
     error->cs.DebugInfo->Spare[0] = 0;
+#endif
     DeleteCriticalSection( &error->cs );
     heap_free( error );
 }
@@ -146,6 +151,7 @@ void WINAPI WsFreeError( WS_ERROR *handle )
 HRESULT WINAPI WsResetError( WS_ERROR *handle )
 {
     struct error *error = (struct error *)handle;
+    HRESULT hr = S_OK;
 
     TRACE( "%p\n", handle );
 
@@ -162,7 +168,8 @@ HRESULT WINAPI WsResetError( WS_ERROR *handle )
     reset_error( error );
 
     LeaveCriticalSection( &error->cs );
-    return S_OK;
+    TRACE( "returning %08x\n", hr );
+    return hr;
 }
 
 /**************************************************************************
@@ -189,6 +196,7 @@ HRESULT WINAPI WsGetErrorProperty( WS_ERROR *handle, WS_ERROR_PROPERTY_ID id, vo
     hr = prop_get( error->prop, error->prop_count, id, buf, size );
 
     LeaveCriticalSection( &error->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -222,14 +230,10 @@ HRESULT WINAPI WsSetErrorProperty( WS_ERROR *handle, WS_ERROR_PROPERTY_ID id, co
         return E_INVALIDARG;
     }
 
-    if (id == WS_ERROR_PROPERTY_LANGID)
-    {
-        LeaveCriticalSection( &error->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    hr = prop_set( error->prop, error->prop_count, id, value, size );
+    if (id == WS_ERROR_PROPERTY_LANGID) hr = WS_E_INVALID_OPERATION;
+    else hr = prop_set( error->prop, error->prop_count, id, value, size );
 
     LeaveCriticalSection( &error->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
