@@ -24,6 +24,7 @@
 #include "webservices.h"
 
 #include "wine/debug.h"
+#include "wine/heap.h"
 #include "wine/list.h"
 #include "webservices_private.h"
 
@@ -87,37 +88,47 @@ int find_string( const struct dictionary *dict, const unsigned char *data, ULONG
 #define MIN_DICTIONARY_SIZE 256
 #define MAX_DICTIONARY_SIZE 2048
 
-static BOOL grow_dict( struct dictionary *dict, ULONG size )
+static HRESULT grow_dict( struct dictionary *dict, ULONG size )
 {
     WS_XML_STRING *tmp;
-    ULONG new_size, *tmp_sorted;
+    ULONG new_size, *tmp_sorted, *tmp_sequence;
 
     assert( !dict->dict.isConst );
-    if (dict->size >= dict->dict.stringCount + size) return TRUE;
-    if (dict->size + size > MAX_DICTIONARY_SIZE) return FALSE;
+    if (dict->size >= dict->dict.stringCount + size) return S_OK;
+    if (dict->size + size > MAX_DICTIONARY_SIZE) return WS_E_QUOTA_EXCEEDED;
 
     if (!dict->dict.strings)
     {
         new_size = max( MIN_DICTIONARY_SIZE, size );
-        if (!(dict->dict.strings = heap_alloc( new_size * sizeof(*dict->dict.strings) ))) return FALSE;
+        if (!(dict->dict.strings = heap_alloc( new_size * sizeof(*dict->dict.strings) ))) return E_OUTOFMEMORY;
         if (!(dict->sorted = heap_alloc( new_size * sizeof(*dict->sorted) )))
         {
             heap_free( dict->dict.strings );
             dict->dict.strings = NULL;
-            return FALSE;
+            return E_OUTOFMEMORY;
+        }
+        if (!(dict->sequence = heap_alloc( new_size * sizeof(*dict->sequence) )))
+        {
+            heap_free( dict->dict.strings );
+            dict->dict.strings = NULL;
+            heap_free( dict->sorted );
+            dict->sorted = NULL;
+            return E_OUTOFMEMORY;
         }
         dict->size = new_size;
-        return TRUE;
+        return S_OK;
     }
 
     new_size = max( dict->size * 2, size );
-    if (!(tmp = heap_realloc( dict->dict.strings, new_size * sizeof(*tmp) ))) return FALSE;
+    if (!(tmp = heap_realloc( dict->dict.strings, new_size * sizeof(*tmp) ))) return E_OUTOFMEMORY;
     dict->dict.strings = tmp;
-    if (!(tmp_sorted = heap_realloc( dict->sorted, new_size * sizeof(*tmp_sorted) ))) return FALSE;
+    if (!(tmp_sorted = heap_realloc( dict->sorted, new_size * sizeof(*tmp_sorted) ))) return E_OUTOFMEMORY;
     dict->sorted = tmp_sorted;
+    if (!(tmp_sequence = heap_realloc( dict->sequence, new_size * sizeof(*tmp_sequence) ))) return E_OUTOFMEMORY;
+    dict->sequence = tmp_sequence;
 
     dict->size = new_size;
-    return TRUE;
+    return S_OK;
 }
 
 void clear_dict( struct dictionary *dict )
@@ -130,14 +141,19 @@ void clear_dict( struct dictionary *dict )
     dict->dict.stringCount = 0;
     heap_free( dict->sorted );
     dict->sorted = NULL;
+    heap_free( dict->sequence );
+    dict->sequence = NULL;
+    dict->current_sequence = 0;
     dict->size = 0;
 }
 
-BOOL insert_string( struct dictionary *dict, unsigned char *data, ULONG len, int i, ULONG *ret_id )
+HRESULT insert_string( struct dictionary *dict, unsigned char *data, ULONG len, int i, ULONG *ret_id )
 {
     ULONG id = dict->dict.stringCount;
+    HRESULT hr;
+
     assert( !dict->dict.isConst );
-    if (!grow_dict( dict, 1 )) return FALSE;
+    if ((hr = grow_dict( dict, 1 )) != S_OK) return hr;
     memmove( &dict->sorted[i] + 1, &dict->sorted[i], (dict->dict.stringCount - i) * sizeof(*dict->sorted) );
     dict->sorted[i] = id;
 
@@ -146,23 +162,16 @@ BOOL insert_string( struct dictionary *dict, unsigned char *data, ULONG len, int
     dict->dict.strings[id].dictionary = &dict->dict;
     dict->dict.strings[id].id         = id;
     dict->dict.stringCount++;
+
+    dict->sequence[id] = dict->current_sequence;
+
     if (ret_id) *ret_id = id;
-    return TRUE;
-}
-
-HRESULT CALLBACK insert_string_cb( void *state, const WS_XML_STRING *str, BOOL *found, ULONG *id, WS_ERROR *error )
-{
-    struct dictionary *dict = state;
-    int index = find_string( dict, str->bytes, str->length, id );
-
-    assert( !dict->dict.isConst );
-    if (index == -1 || insert_string( dict, str->bytes, str->length, index, id )) *found = TRUE;
-    else *found = FALSE;
     return S_OK;
 }
 
 HRESULT add_xml_string( WS_XML_STRING *str )
 {
+    HRESULT hr = S_OK;
     int index;
     ULONG id;
 
@@ -172,17 +181,13 @@ HRESULT add_xml_string( WS_XML_STRING *str )
     {
         heap_free( str->bytes );
         *str = dict_builtin.dict.strings[id];
-        LeaveCriticalSection( &dict_cs );
-        return S_OK;
     }
-    if (insert_string( &dict_builtin, str->bytes, str->length, index, &id ))
+    else if ((hr = insert_string( &dict_builtin, str->bytes, str->length, index, &id )) == S_OK)
     {
         *str = dict_builtin.dict.strings[id];
-        LeaveCriticalSection( &dict_cs );
-        return S_OK;
     }
     LeaveCriticalSection( &dict_cs );
-    return WS_E_QUOTA_EXCEEDED;
+    return hr;
 }
 
 WS_XML_STRING *alloc_xml_string( const unsigned char *data, ULONG len )
@@ -210,10 +215,11 @@ void free_xml_string( WS_XML_STRING *str )
     heap_free( str );
 }
 
-WS_XML_STRING *dup_xml_string( const WS_XML_STRING *src )
+WS_XML_STRING *dup_xml_string( const WS_XML_STRING *src, BOOL use_static_dict )
 {
     WS_XML_STRING *ret;
     unsigned char *data;
+    HRESULT hr;
     int index;
     ULONG id;
 
@@ -223,6 +229,11 @@ WS_XML_STRING *dup_xml_string( const WS_XML_STRING *src )
         *ret = *src;
         return ret;
     }
+    if (use_static_dict && (index = find_string( &dict_builtin_static, src->bytes, src->length, &id )) == -1)
+    {
+        *ret = dict_builtin_static.dict.strings[id];
+        return ret;
+    }
     EnterCriticalSection( &dict_cs );
     if ((index = find_string( &dict_builtin, src->bytes, src->length, &id )) == -1)
     {
@@ -237,7 +248,7 @@ WS_XML_STRING *dup_xml_string( const WS_XML_STRING *src )
         return NULL;
     }
     memcpy( data, src->bytes, src->length );
-    if (insert_string( &dict_builtin, data, src->length, index, &id ))
+    if ((hr = insert_string( &dict_builtin, data, src->length, index, &id )) == S_OK)
     {
         *ret = dict_builtin.dict.strings[id];
         LeaveCriticalSection( &dict_cs );
@@ -780,7 +791,7 @@ static const ULONG dict_sorted[] =
 const struct dictionary dict_builtin_static =
 {
     {{0xf93578f8,0x5852,0x4eb7,{0xa6,0xfc,0xe7,0x2b,0xb7,0x1d,0xb6,0x22}},
-     (WS_XML_STRING *)dict_strings, sizeof(dict_strings)/sizeof(dict_strings[0]), TRUE},
+     (WS_XML_STRING *)dict_strings, ARRAY_SIZE( dict_strings ), TRUE},
     (ULONG *)dict_sorted
 };
 
