@@ -162,26 +162,6 @@ static INT X11DRV_GetDeviceCaps( PHYSDEV dev, INT cap )
 {
     switch(cap)
     {
-    case HORZRES:
-    {
-        RECT primary_rect = get_primary_monitor_rect();
-        return primary_rect.right - primary_rect.left;
-    }
-    case VERTRES:
-    {
-        RECT primary_rect = get_primary_monitor_rect();
-        return primary_rect.bottom - primary_rect.top;
-    }
-    case DESKTOPHORZRES:
-    {
-        RECT virtual_rect = get_virtual_screen_rect();
-        return virtual_rect.right - virtual_rect.left;
-    }
-    case DESKTOPVERTRES:
-    {
-        RECT virtual_rect = get_virtual_screen_rect();
-        return virtual_rect.bottom - virtual_rect.top;
-    }
     case BITSPIXEL:
         return screen_bpp;
     case SIZEPALETTE:
@@ -346,6 +326,21 @@ static struct opengl_funcs * X11DRV_wine_get_wgl_driver( PHYSDEV dev, UINT versi
     return ret;
 }
 
+/**********************************************************************
+ *           X11DRV_wine_get_vulkan_driver
+ */
+static const struct vulkan_funcs * X11DRV_wine_get_vulkan_driver( PHYSDEV dev, UINT version )
+{
+    const struct vulkan_funcs *ret;
+
+    if (!(ret = get_vulkan_driver( version )))
+    {
+        dev = GET_NEXT_PHYSDEV( dev, wine_get_vulkan_driver );
+        ret = dev->funcs->wine_get_vulkan_driver( dev, version );
+    }
+    return ret;
+}
+
 
 static const struct gdi_dc_funcs x11drv_funcs =
 {
@@ -476,6 +471,7 @@ static const struct gdi_dc_funcs x11drv_funcs =
     X11DRV_UnrealizePalette,            /* pUnrealizePalette */
     NULL,                               /* pWidenPath */
     X11DRV_wine_get_wgl_driver,         /* wine_get_wgl_driver */
+    X11DRV_wine_get_vulkan_driver,      /* wine_get_vulkan_driver */
     GDI_PRIORITY_GRAPHICS_DRV           /* priority */
 };
 
