@@ -140,15 +140,6 @@ typedef struct _FILE_VALID_DATA_LENGTH_INFORMATION
   LARGE_INTEGER ValidDataLength;
 } FILE_VALID_DATA_LENGTH_INFORMATION, *PFILE_VALID_DATA_LENGTH_INFORMATION;
 
-typedef struct _KWAIT_BLOCK {
-    LIST_ENTRY WaitListEntry;
-    struct _KTHREAD *RESTRICTED_POINTER Thread;
-    PVOID Object;
-    struct _KWAIT_BLOCK *RESTRICTED_POINTER NextWaitBlock;
-    USHORT WaitKey;
-    USHORT WaitType;
-} KWAIT_BLOCK, *PKWAIT_BLOCK, *RESTRICTED_POINTER PRKWAIT_BLOCK;
-
 typedef struct _RTL_BALANCED_LINKS {
     struct _RTL_BALANCED_LINKS *Parent;
     struct _RTL_BALANCED_LINKS *LeftChild;
