@@ -561,18 +561,18 @@ HRESULT WINAPI VarDecNeg(const DECIMAL*,DECIMAL*);
 HRESULT WINAPI VarDecRound(const DECIMAL*,int,DECIMAL*);
 HRESULT WINAPI VarDecSub(const DECIMAL*,const DECIMAL*,DECIMAL*);
 
-HRESULT WINAPI VarCyAbs(const CY,CY*);
-HRESULT WINAPI VarCyAdd(const CY,const CY,CY*);
-HRESULT WINAPI VarCyCmp(const CY,const CY);
-HRESULT WINAPI VarCyCmpR8(const CY,DOUBLE);
-HRESULT WINAPI VarCyFix(const CY,CY*);
-HRESULT WINAPI VarCyInt(const CY,CY*);
-HRESULT WINAPI VarCyMul(const CY,CY,CY*);
-HRESULT WINAPI VarCyMulI4(const CY,LONG,CY*);
-HRESULT WINAPI VarCyMulI8(const CY,LONG64,CY*);
-HRESULT WINAPI VarCyNeg(const CY,CY*);
-HRESULT WINAPI VarCyRound(const CY,INT,CY*);
-HRESULT WINAPI VarCySub(const CY,const CY,CY*);
+HRESULT WINAPI VarCyAbs(CY,CY*);
+HRESULT WINAPI VarCyAdd(CY,CY,CY*);
+HRESULT WINAPI VarCyCmp(CY,CY);
+HRESULT WINAPI VarCyCmpR8(CY,DOUBLE);
+HRESULT WINAPI VarCyFix(CY,CY*);
+HRESULT WINAPI VarCyInt(CY,CY*);
+HRESULT WINAPI VarCyMul(CY,CY,CY*);
+HRESULT WINAPI VarCyMulI4(CY,LONG,CY*);
+HRESULT WINAPI VarCyMulI8(CY,LONG64,CY*);
+HRESULT WINAPI VarCyNeg(CY,CY*);
+HRESULT WINAPI VarCyRound(CY,INT,CY*);
+HRESULT WINAPI VarCySub(CY,CY,CY*);
 
 HRESULT WINAPI VarAdd(LPVARIANT,LPVARIANT,LPVARIANT);
 HRESULT WINAPI VarAnd(LPVARIANT,LPVARIANT,LPVARIANT);
