@@ -1227,6 +1227,13 @@ union codeview_fieldtype
 #define LF_COMPLEX128           0x800f
 #define LF_VARSTRING            0x8010
 
+/* symtype e.g. for public_vx.symtype */
+#define SYMTYPE_NONE            0x0000
+#define SYMTYPE_CODE            0x0001
+#define SYMTYPE_FUNCTION        0x0002
+#define SYMTYPE_MANAGED         0x0004
+#define SYMTYPE_MSIL            0x0008
+
 /* ======================================== *
  *            Symbol information
  * ======================================== */
@@ -1348,6 +1355,16 @@ union codeview_symbol
         char                    name[1];
     } proc_v3;
 
+    struct
+    {
+        short int               len;
+        short int               id;
+        unsigned int            offset;
+        unsigned short          segment;
+        unsigned short          symtype;
+        struct p_string         p_name;
+    } public_v1;
+
     struct
     {
         short int               len;
