@@ -86,6 +86,7 @@ typedef struct
 typedef struct
 {
     int           nb_args;
+    int           args_str_offset;
     enum arg_type args[MAX_ARGUMENTS];
 } ORD_FUNCTION;
 
@@ -360,7 +361,9 @@ extern struct strarray cc_command;
 extern struct strarray ld_command;
 extern struct strarray nm_command;
 extern char *cpu_option;
+extern char *fpu_option;
 extern char *arch_option;
+extern const char *float_abi_option;
 extern int thumb_mode;
 extern int needs_get_pc_thunk;
 
