@@ -212,6 +212,12 @@ static const struct
     const char *name;
 } languages[] =
 {
+    { LANG_AFRIKAANS,      SUBLANG_NEUTRAL,                     "af" },
+    { LANG_AFRIKAANS,      SUBLANG_AFRIKAANS_SOUTH_AFRICA,      "af_ZA" },
+    { LANG_ALBANIAN,       SUBLANG_NEUTRAL,                     "sq" },
+    { LANG_ALBANIAN,       SUBLANG_ALBANIAN_ALBANIA,            "sq_AL" },
+    { LANG_AMHARIC,        SUBLANG_NEUTRAL,                     "am" },
+    { LANG_AMHARIC,        SUBLANG_AMHARIC_ETHIOPIA,            "am_ET" },
     { LANG_ARABIC,         SUBLANG_NEUTRAL,                     "ar" },
     { LANG_ARABIC,         SUBLANG_ARABIC_SAUDI_ARABIA,         "ar_SA" },
     { LANG_ARABIC,         SUBLANG_ARABIC_IRAQ,                 "ar_IQ" },
@@ -229,6 +235,22 @@ static const struct
     { LANG_ARABIC,         SUBLANG_ARABIC_UAE,                  "ar_AE" },
     { LANG_ARABIC,         SUBLANG_ARABIC_BAHRAIN,              "ar_BH" },
     { LANG_ARABIC,         SUBLANG_ARABIC_QATAR,                "ar_QA" },
+    { LANG_ARMENIAN,       SUBLANG_NEUTRAL,                     "hy" },
+    { LANG_ARMENIAN,       SUBLANG_ARMENIAN_ARMENIA,            "hy_AM" },
+    { LANG_ASSAMESE,       SUBLANG_NEUTRAL,                     "as" },
+    { LANG_ASSAMESE,       SUBLANG_ASSAMESE_INDIA,              "as_IN" },
+    { LANG_AZERBAIJANI,    SUBLANG_NEUTRAL,                     "az" },
+    { LANG_AZERBAIJANI,    SUBLANG_AZERBAIJANI_AZERBAIJAN_LATIN,"az_AZ@latin" },
+    { LANG_AZERBAIJANI,    SUBLANG_AZERBAIJANI_AZERBAIJAN_CYRILLIC, "az_AZ@cyrillic" },
+    { LANG_BASQUE,         SUBLANG_NEUTRAL,                     "eu" },
+    { LANG_BASQUE,         SUBLANG_BASQUE_BASQUE,               "eu_ES" },
+    { LANG_BELARUSIAN,     SUBLANG_NEUTRAL,                     "be" },
+    { LANG_BELARUSIAN,     SUBLANG_BELARUSIAN_BELARUS,          "be_BY" },
+    { LANG_BENGALI,        SUBLANG_NEUTRAL,                     "bn" },
+    { LANG_BENGALI,        SUBLANG_BENGALI_INDIA,               "bn_IN" },
+    { LANG_BENGALI,        SUBLANG_BENGALI_BANGLADESH,          "bn_BD" },
+    { LANG_BRETON,         SUBLANG_NEUTRAL,                     "br" },
+    { LANG_BRETON,         SUBLANG_BRETON_FRANCE,               "br_FR" },
     { LANG_BULGARIAN,      SUBLANG_NEUTRAL,                     "bg" },
     { LANG_BULGARIAN,      SUBLANG_BULGARIAN_BULGARIA,          "bg_BG" },
     { LANG_CATALAN,        SUBLANG_NEUTRAL,                     "ca" },
@@ -243,14 +265,12 @@ static const struct
     { LANG_CZECH,          SUBLANG_CZECH_CZECH_REPUBLIC,        "cs_CZ" },
     { LANG_DANISH,         SUBLANG_NEUTRAL,                     "da" },
     { LANG_DANISH,         SUBLANG_DANISH_DENMARK,              "da_DK" },
-    { LANG_GERMAN,         SUBLANG_NEUTRAL,                     "de" },
-    { LANG_GERMAN,         SUBLANG_GERMAN,                      "de_DE" },
-    { LANG_GERMAN,         SUBLANG_GERMAN_SWISS,                "de_CH" },
-    { LANG_GERMAN,         SUBLANG_GERMAN_AUSTRIAN,             "de_AT" },
-    { LANG_GERMAN,         SUBLANG_GERMAN_LUXEMBOURG,           "de_LU" },
-    { LANG_GERMAN,         SUBLANG_GERMAN_LIECHTENSTEIN,        "de_LI" },
-    { LANG_GREEK,          SUBLANG_NEUTRAL,                     "el" },
-    { LANG_GREEK,          SUBLANG_GREEK_GREECE,                "el_GR" },
+    { LANG_DIVEHI,         SUBLANG_NEUTRAL,                     "dv" },
+    { LANG_DIVEHI,         SUBLANG_DIVEHI_MALDIVES,             "dv_MV" },
+    { LANG_DUTCH,          SUBLANG_NEUTRAL,                     "nl" },
+    { LANG_DUTCH,          SUBLANG_DUTCH,                       "nl_NL" },
+    { LANG_DUTCH,          SUBLANG_DUTCH_BELGIAN,               "nl_BE" },
+    { LANG_DUTCH,          SUBLANG_DUTCH_SURINAM,               "nl_SR" },
     { LANG_ENGLISH,        SUBLANG_NEUTRAL,                     "en" },
     { LANG_ENGLISH,        SUBLANG_ENGLISH_US,                  "en_US" },
     { LANG_ENGLISH,        SUBLANG_ENGLISH_UK,                  "en_GB" },
@@ -265,6 +285,152 @@ static const struct
     { LANG_ENGLISH,        SUBLANG_ENGLISH_TRINIDAD,            "en_TT" },
     { LANG_ENGLISH,        SUBLANG_ENGLISH_ZIMBABWE,            "en_ZW" },
     { LANG_ENGLISH,        SUBLANG_ENGLISH_PHILIPPINES,         "en_PH" },
+    { LANG_ENGLISH,        SUBLANG_ENGLISH_INDIA,               "en_IN" },
+    { LANG_ENGLISH,        SUBLANG_ENGLISH_MALAYSIA,            "en_MY" },
+    { LANG_ENGLISH,        SUBLANG_ENGLISH_SINGAPORE,           "en_SG" },
+    { LANG_ESTONIAN,       SUBLANG_NEUTRAL,                     "et" },
+    { LANG_ESTONIAN,       SUBLANG_ESTONIAN_ESTONIA,            "et_EE" },
+    { LANG_FAEROESE,       SUBLANG_NEUTRAL,                     "fo" },
+    { LANG_FAEROESE,       SUBLANG_FAEROESE_FAROE_ISLANDS,      "fo_FO" },
+    { LANG_FILIPINO,       SUBLANG_NEUTRAL,                     "fil" },
+    { LANG_FILIPINO,       SUBLANG_FILIPINO_PHILIPPINES,        "fil_PH" },
+    { LANG_FINNISH,        SUBLANG_NEUTRAL,                     "fi" },
+    { LANG_FINNISH,        SUBLANG_FINNISH_FINLAND,             "fi_FI" },
+    { LANG_FRENCH,         SUBLANG_NEUTRAL,                     "fr" },
+    { LANG_FRENCH,         SUBLANG_FRENCH,                      "fr_FR" },
+    { LANG_FRENCH,         SUBLANG_FRENCH_BELGIAN,              "fr_BE" },
+    { LANG_FRENCH,         SUBLANG_FRENCH_CANADIAN,             "fr_CA" },
+    { LANG_FRENCH,         SUBLANG_FRENCH_SWISS,                "fr_CH" },
+    { LANG_FRENCH,         SUBLANG_FRENCH_LUXEMBOURG,           "fr_LU" },
+    { LANG_FRENCH,         SUBLANG_FRENCH_MONACO,               "fr_MC" },
+    { LANG_GALICIAN,       SUBLANG_NEUTRAL,                     "gl" },
+    { LANG_GALICIAN,       SUBLANG_GALICIAN_GALICIAN,           "gl_ES" },
+    { LANG_GEORGIAN,       SUBLANG_NEUTRAL,                     "ka" },
+    { LANG_GEORGIAN,       SUBLANG_GEORGIAN_GEORGIA,            "ka_GE" },
+    { LANG_GERMAN,         SUBLANG_NEUTRAL,                     "de" },
+    { LANG_GERMAN,         SUBLANG_GERMAN,                      "de_DE" },
+    { LANG_GERMAN,         SUBLANG_GERMAN_SWISS,                "de_CH" },
+    { LANG_GERMAN,         SUBLANG_GERMAN_AUSTRIAN,             "de_AT" },
+    { LANG_GERMAN,         SUBLANG_GERMAN_LUXEMBOURG,           "de_LU" },
+    { LANG_GERMAN,         SUBLANG_GERMAN_LIECHTENSTEIN,        "de_LI" },
+    { LANG_GREEK,          SUBLANG_NEUTRAL,                     "el" },
+    { LANG_GREEK,          SUBLANG_GREEK_GREECE,                "el_GR" },
+    { LANG_GUJARATI,       SUBLANG_NEUTRAL,                     "gu" },
+    { LANG_GUJARATI,       SUBLANG_GUJARATI_INDIA,              "gu_IN" },
+    { LANG_HAUSA,          SUBLANG_NEUTRAL,                     "ha" },
+    { LANG_HAUSA,          SUBLANG_HAUSA_NIGERIA,               "ha_NG" },
+    { LANG_HAWAIIAN,       SUBLANG_NEUTRAL,                     "haw" },
+    { LANG_HAWAIIAN,       SUBLANG_HAWAIIAN_US,                 "haw_US" },
+    { LANG_HEBREW,         SUBLANG_NEUTRAL,                     "he" },
+    { LANG_HEBREW,         SUBLANG_HEBREW_ISRAEL,               "he_IL" },
+    { LANG_HINDI,          SUBLANG_NEUTRAL,                     "hi" },
+    { LANG_HINDI,          SUBLANG_HINDI_INDIA,                 "hi_IN" },
+    { LANG_HUNGARIAN,      SUBLANG_NEUTRAL,                     "hu" },
+    { LANG_HUNGARIAN,      SUBLANG_HUNGARIAN_HUNGARY,           "hu_HU" },
+    { LANG_ICELANDIC,      SUBLANG_NEUTRAL,                     "is" },
+    { LANG_ICELANDIC,      SUBLANG_ICELANDIC_ICELAND,           "is_IS" },
+    { LANG_IGBO,           SUBLANG_NEUTRAL,                     "ig" },
+    { LANG_IGBO,           SUBLANG_IGBO_NIGERIA,                "ig_NG" },
+    { LANG_INDONESIAN,     SUBLANG_NEUTRAL,                     "id" },
+    { LANG_INDONESIAN,     SUBLANG_INDONESIAN_INDONESIA,        "id_ID" },
+    { LANG_INUKTITUT,      SUBLANG_NEUTRAL,                     "iu" },
+    { LANG_INUKTITUT,      SUBLANG_INUKTITUT_CANADA,            "iu_CA" },
+    { LANG_IRISH,          SUBLANG_NEUTRAL,                     "ga" },
+    { LANG_IRISH,          SUBLANG_IRISH_IRELAND,               "ga_IE" },
+    { LANG_ITALIAN,        SUBLANG_NEUTRAL,                     "it" },
+    { LANG_ITALIAN,        SUBLANG_ITALIAN,                     "it_IT" },
+    { LANG_ITALIAN,        SUBLANG_ITALIAN_SWISS,               "it_CH" },
+    { LANG_JAPANESE,       SUBLANG_NEUTRAL,                     "ja" },
+    { LANG_JAPANESE,       SUBLANG_JAPANESE_JAPAN,              "ja_JP" },
+    { LANG_KANNADA,        SUBLANG_NEUTRAL,                     "kn" },
+    { LANG_KANNADA,        SUBLANG_KANNADA_INDIA,               "kn_IN" },
+    { LANG_KAZAK,          SUBLANG_NEUTRAL,                     "kk" },
+    { LANG_KAZAK,          SUBLANG_KAZAK_KAZAKHSTAN,            "kk_KZ" },
+    { LANG_KHMER,          SUBLANG_NEUTRAL,                     "km" },
+    { LANG_KHMER,          SUBLANG_KHMER_CAMBODIA,              "km_KH" },
+    { LANG_KINYARWANDA,    SUBLANG_NEUTRAL,                     "rw" },
+    { LANG_KINYARWANDA,    SUBLANG_KINYARWANDA_RWANDA,          "rw_RW" },
+    { LANG_KONKANI,        SUBLANG_NEUTRAL,                     "kok" },
+    { LANG_KONKANI,        SUBLANG_KONKANI_INDIA,               "kok_IN" },
+    { LANG_KOREAN,         SUBLANG_NEUTRAL,                     "ko" },
+    { LANG_KOREAN,         SUBLANG_KOREAN,                      "ko_KR" },
+    { LANG_KYRGYZ,         SUBLANG_NEUTRAL,                     "ky" },
+    { LANG_KYRGYZ,         SUBLANG_KYRGYZ_KYRGYZSTAN,           "ky_KG" },
+    { LANG_LAO,            SUBLANG_NEUTRAL,                     "lo" },
+    { LANG_LAO,            SUBLANG_LAO_LAO,                     "lo_LA" },
+    { LANG_LATVIAN,        SUBLANG_NEUTRAL,                     "lv" },
+    { LANG_LATVIAN,        SUBLANG_LATVIAN_LATVIA,              "lv_LV" },
+    { LANG_LITHUANIAN,     SUBLANG_NEUTRAL,                     "lt" },
+    { LANG_LITHUANIAN,     SUBLANG_LITHUANIAN,                  "lt_LT" },
+    { LANG_MACEDONIAN,     SUBLANG_NEUTRAL,                     "mk" },
+    { LANG_MACEDONIAN,     SUBLANG_MACEDONIAN_MACEDONIA,        "mk_MK" },
+    { LANG_MALAY,          SUBLANG_NEUTRAL,                     "ms" },
+    { LANG_MALAY,          SUBLANG_MALAY_MALAYSIA,              "ms_MY" },
+    { LANG_MALAY,          SUBLANG_MALAY_BRUNEI_DARUSSALAM,     "ms_BN" },
+    { LANG_MALAYALAM,      SUBLANG_NEUTRAL,                     "ml" },
+    { LANG_MALAYALAM,      SUBLANG_MALAYALAM_INDIA,             "ml_IN" },
+    { LANG_MALTESE,        SUBLANG_NEUTRAL,                     "mt" },
+    { LANG_MALTESE,        SUBLANG_MALTESE_MALTA,               "mt_MT" },
+    { LANG_MARATHI,        SUBLANG_NEUTRAL,                     "mr" },
+    { LANG_MARATHI,        SUBLANG_MARATHI_INDIA,               "mr_IN" },
+    { LANG_MONGOLIAN,      SUBLANG_NEUTRAL,                     "mn" },
+    { LANG_MONGOLIAN,      SUBLANG_MONGOLIAN_CYRILLIC_MONGOLIA, "mn_MN" },
+    { LANG_MONGOLIAN,      SUBLANG_MONGOLIAN_PRC,               "mn_CN" },
+    { LANG_NEPALI,         SUBLANG_NEUTRAL,                     "ne" },
+    { LANG_NEPALI,         SUBLANG_NEPALI_NEPAL,                "ne_NP" },
+    { LANG_NEPALI,         SUBLANG_NEPALI_INDIA,                "ne_IN" },
+    { LANG_NORWEGIAN,      SUBLANG_NORWEGIAN_BOKMAL,            "nb_NO" },
+    { LANG_NORWEGIAN,      SUBLANG_NORWEGIAN_NYNORSK,           "nn_NO" },
+    { LANG_ODIA,           SUBLANG_NEUTRAL,                     "or" },
+    { LANG_ODIA,           SUBLANG_ODIA_INDIA,                  "or_IN" },
+    { LANG_PASHTO,         SUBLANG_NEUTRAL,                     "ps" },
+    { LANG_PASHTO,         SUBLANG_PASHTO_AFGHANISTAN,          "ps_AF" },
+    { LANG_PERSIAN,        SUBLANG_NEUTRAL,                     "fa" },
+    { LANG_PERSIAN,        SUBLANG_PERSIAN_IRAN,                "fa_IR" },
+    { LANG_POLISH,         SUBLANG_NEUTRAL,                     "pl" },
+    { LANG_POLISH,         SUBLANG_POLISH_POLAND,               "pl_PL" },
+    { LANG_PORTUGUESE,     SUBLANG_NEUTRAL,                     "pt" },
+    { LANG_PORTUGUESE,     SUBLANG_PORTUGUESE_BRAZILIAN,        "pt_BR" },
+    { LANG_PORTUGUESE,     SUBLANG_PORTUGUESE_PORTUGAL,         "pt_PT" },
+    { LANG_PUNJABI,        SUBLANG_NEUTRAL,                     "pa" },
+    { LANG_PUNJABI,        SUBLANG_PUNJABI_INDIA,               "pa_IN" },
+    { LANG_PUNJABI,        SUBLANG_PUNJABI_PAKISTAN,            "pa_PK" },
+    { LANG_ROMANIAN,       SUBLANG_NEUTRAL,                     "ro" },
+    { LANG_ROMANIAN,       SUBLANG_ROMANIAN_ROMANIA,            "ro_RO" },
+    { LANG_ROMANSH,        SUBLANG_NEUTRAL,                     "rm" },
+    { LANG_ROMANSH,        SUBLANG_ROMANSH_SWITZERLAND,         "rm_CH" },
+    { LANG_RUSSIAN,        SUBLANG_NEUTRAL,                     "ru" },
+    { LANG_RUSSIAN,        SUBLANG_RUSSIAN_RUSSIA,              "ru_RU" },
+    { LANG_SAMI,           SUBLANG_NEUTRAL,                     "se" },
+    { LANG_SAMI,           SUBLANG_SAMI_NORTHERN_NORWAY,        "se_NO" },
+    { LANG_SAMI,           SUBLANG_SAMI_NORTHERN_SWEDEN,        "se_SE" },
+    { LANG_SAMI,           SUBLANG_SAMI_NORTHERN_FINLAND,       "se_FI" },
+    { LANG_SANSKRIT,       SUBLANG_NEUTRAL,                     "sa" },
+    { LANG_SANSKRIT,       SUBLANG_SANSKRIT_INDIA,              "sa_IN" },
+    { LANG_SCOTTISH_GAELIC,SUBLANG_NEUTRAL,                     "gd" },
+    { LANG_SCOTTISH_GAELIC,SUBLANG_SCOTTISH_GAELIC,             "gd_GB" },
+    /* LANG_SERBIAN/LANG_CROATIAN/LANG_BOSNIAN are the same */
+    { LANG_SERBIAN,        SUBLANG_NEUTRAL,                     "hr" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_CROATIA,             "hr_HR" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_LATIN,               "sr_RS@latin" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_CYRILLIC,            "sr_RS@cyrillic" },
+    { LANG_SERBIAN,        SUBLANG_CROATIAN_BOSNIA_HERZEGOVINA_LATIN,   "hr_BA@latin" },
+    { LANG_SERBIAN,        SUBLANG_BOSNIAN_BOSNIA_HERZEGOVINA_LATIN,    "bs_BA@latin" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_BOSNIA_HERZEGOVINA_LATIN,    "sr_BA@latin" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_BOSNIA_HERZEGOVINA_CYRILLIC, "sr_BA@cyrillic" },
+    { LANG_SERBIAN,        SUBLANG_BOSNIAN_BOSNIA_HERZEGOVINA_CYRILLIC, "bs_BA@cyrillic" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_SERBIA_LATIN,        "sr_RS@latin" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_SERBIA_CYRILLIC,     "sr_RS@cyrillic" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_MONTENEGRO_LATIN,    "sr_ME@latin" },
+    { LANG_SERBIAN,        SUBLANG_SERBIAN_MONTENEGRO_CYRILLIC, "sr_ME@cyrillic" },
+    { LANG_SINHALESE,      SUBLANG_NEUTRAL,                     "si" },
+    { LANG_SINHALESE,      SUBLANG_SINHALESE_SRI_LANKA,         "si_LK" },
+    { LANG_SLOVAK,         SUBLANG_NEUTRAL,                     "sk" },
+    { LANG_SLOVAK,         SUBLANG_SLOVAK_SLOVAKIA,             "sk_SK" },
+    { LANG_SLOVENIAN,      SUBLANG_NEUTRAL,                     "sl" },
+    { LANG_SLOVENIAN,      SUBLANG_SLOVENIAN_SLOVENIA,          "sl_SI" },
+    { LANG_SOTHO,          SUBLANG_NEUTRAL,                     "nso" },
+    { LANG_SOTHO,          SUBLANG_SOTHO_NORTHERN_SOUTH_AFRICA, "nso_ZA" },
     { LANG_SPANISH,        SUBLANG_NEUTRAL,                     "es" },
     { LANG_SPANISH,        SUBLANG_SPANISH,                     "es_ES" },
     { LANG_SPANISH,        SUBLANG_SPANISH_MEXICAN,             "es_MX" },
@@ -286,143 +452,53 @@ static const struct
     { LANG_SPANISH,        SUBLANG_SPANISH_HONDURAS,            "es_HN" },
     { LANG_SPANISH,        SUBLANG_SPANISH_NICARAGUA,           "es_NI" },
     { LANG_SPANISH,        SUBLANG_SPANISH_PUERTO_RICO,         "es_PR" },
-    { LANG_FINNISH,        SUBLANG_NEUTRAL,                     "fi" },
-    { LANG_FINNISH,        SUBLANG_FINNISH_FINLAND,             "fi_FI" },
-    { LANG_FRENCH,         SUBLANG_NEUTRAL,                     "fr" },
-    { LANG_FRENCH,         SUBLANG_FRENCH,                      "fr_FR" },
-    { LANG_FRENCH,         SUBLANG_FRENCH_BELGIAN,              "fr_BE" },
-    { LANG_FRENCH,         SUBLANG_FRENCH_CANADIAN,             "fr_CA" },
-    { LANG_FRENCH,         SUBLANG_FRENCH_SWISS,                "fr_CH" },
-    { LANG_FRENCH,         SUBLANG_FRENCH_LUXEMBOURG,           "fr_LU" },
-    { LANG_FRENCH,         SUBLANG_FRENCH_MONACO,               "fr_MC" },
-    { LANG_HEBREW,         SUBLANG_NEUTRAL,                     "he" },
-    { LANG_HEBREW,         SUBLANG_HEBREW_ISRAEL,               "he_IL" },
-    { LANG_HUNGARIAN,      SUBLANG_NEUTRAL,                     "hu" },
-    { LANG_HUNGARIAN,      SUBLANG_HUNGARIAN_HUNGARY,           "hu_HU" },
-    { LANG_ICELANDIC,      SUBLANG_NEUTRAL,                     "is" },
-    { LANG_ICELANDIC,      SUBLANG_ICELANDIC_ICELAND,           "is_IS" },
-    { LANG_IRISH,          SUBLANG_NEUTRAL,                     "ga" },
-    { LANG_IRISH,          SUBLANG_IRISH_IRELAND,               "ga_IE" },
-    { LANG_ITALIAN,        SUBLANG_NEUTRAL,                     "it" },
-    { LANG_ITALIAN,        SUBLANG_ITALIAN,                     "it_IT" },
-    { LANG_ITALIAN,        SUBLANG_ITALIAN_SWISS,               "it_CH" },
-    { LANG_JAPANESE,       SUBLANG_NEUTRAL,                     "ja" },
-    { LANG_JAPANESE,       SUBLANG_JAPANESE_JAPAN,              "ja_JP" },
-    { LANG_KOREAN,         SUBLANG_NEUTRAL,                     "ko" },
-    { LANG_KOREAN,         SUBLANG_KOREAN,                      "ko_KR" },
-    { LANG_DUTCH,          SUBLANG_NEUTRAL,                     "nl" },
-    { LANG_DUTCH,          SUBLANG_DUTCH,                       "nl_NL" },
-    { LANG_DUTCH,          SUBLANG_DUTCH_BELGIAN,               "nl_BE" },
-    { LANG_DUTCH,          SUBLANG_DUTCH_SURINAM,               "nl_SR" },
-    { LANG_NORWEGIAN,      SUBLANG_NORWEGIAN_BOKMAL,            "nb_NO" },
-    { LANG_NORWEGIAN,      SUBLANG_NORWEGIAN_NYNORSK,           "nn_NO" },
-    { LANG_POLISH,         SUBLANG_NEUTRAL,                     "pl" },
-    { LANG_POLISH,         SUBLANG_POLISH_POLAND,               "pl_PL" },
-    { LANG_PORTUGUESE,     SUBLANG_NEUTRAL,                     "pt" },
-    { LANG_PORTUGUESE,     SUBLANG_PORTUGUESE_BRAZILIAN,        "pt_BR" },
-    { LANG_PORTUGUESE,     SUBLANG_PORTUGUESE_PORTUGAL,         "pt_PT" },
-    { LANG_ROMANSH,        SUBLANG_NEUTRAL,                     "rm" },
-    { LANG_ROMANSH,        SUBLANG_ROMANSH_SWITZERLAND,         "rm_CH" },
-    { LANG_ROMANIAN,       SUBLANG_NEUTRAL,                     "ro" },
-    { LANG_ROMANIAN,       SUBLANG_ROMANIAN_ROMANIA,            "ro_RO" },
-    { LANG_RUSSIAN,        SUBLANG_NEUTRAL,                     "ru" },
-    { LANG_RUSSIAN,        SUBLANG_RUSSIAN_RUSSIA,              "ru_RU" },
-    { LANG_SCOTTISH_GAELIC,SUBLANG_NEUTRAL,                     "gd" },
-    { LANG_SCOTTISH_GAELIC,SUBLANG_SCOTTISH_GAELIC,             "gd_GB" },
-    { LANG_SERBIAN,        SUBLANG_NEUTRAL,                     "hr" },
-    { LANG_SERBIAN,        SUBLANG_SERBIAN_CROATIA,             "hr_HR" },
-    { LANG_SERBIAN,        SUBLANG_SERBIAN_LATIN,               "sr_RS@latin" },
-    { LANG_SERBIAN,        SUBLANG_SERBIAN_CYRILLIC,            "sr_RS@cyrillic" },
-    { LANG_SLOVAK,         SUBLANG_NEUTRAL,                     "sk" },
-    { LANG_SLOVAK,         SUBLANG_SLOVAK_SLOVAKIA,             "sk_SK" },
-    { LANG_ALBANIAN,       SUBLANG_NEUTRAL,                     "sq" },
-    { LANG_ALBANIAN,       SUBLANG_ALBANIAN_ALBANIA,            "sq_AL" },
+    { LANG_SPANISH,        SUBLANG_SPANISH_US,                  "es_US" },
+    { LANG_SWAHILI,        SUBLANG_NEUTRAL,                     "sw" },
+    { LANG_SWAHILI,        SUBLANG_SWAHILI_KENYA,               "sw_KE" },
     { LANG_SWEDISH,        SUBLANG_NEUTRAL,                     "sv" },
     { LANG_SWEDISH,        SUBLANG_SWEDISH_SWEDEN,              "sv_SE" },
     { LANG_SWEDISH,        SUBLANG_SWEDISH_FINLAND,             "sv_FI" },
+    { LANG_SYRIAC,         SUBLANG_NEUTRAL,                     "syr" },
+    { LANG_SYRIAC,         SUBLANG_SYRIAC_SYRIA,                "syr_SY" },
+    { LANG_TAJIK,          SUBLANG_NEUTRAL,                     "tg" },
+    { LANG_TAJIK,          SUBLANG_TAJIK_TAJIKISTAN,            "tg_TJ" },
+    { LANG_TAMIL,          SUBLANG_NEUTRAL,                     "ta" },
+    { LANG_TAMIL,          SUBLANG_TAMIL_INDIA,                 "ta_IN" },
+    { LANG_TATAR,          SUBLANG_NEUTRAL,                     "tt" },
+    { LANG_TATAR,          SUBLANG_TATAR_RUSSIA,                "tt_TA" },
+    { LANG_TELUGU,         SUBLANG_NEUTRAL,                     "te" },
+    { LANG_TELUGU,         SUBLANG_TELUGU_INDIA,                "te_IN" },
     { LANG_THAI,           SUBLANG_NEUTRAL,                     "th" },
     { LANG_THAI,           SUBLANG_THAI_THAILAND,               "th_TH" },
+    { LANG_TIGRINYA,       SUBLANG_NEUTRAL,                     "ti" },
+    { LANG_TIGRINYA,       SUBLANG_TIGRINYA_ETHIOPIA,           "ti_ET" },
+    { LANG_TIGRINYA,       SUBLANG_TIGRINYA_ERITREA,            "ti_ER" },
+    { LANG_TSWANA,         SUBLANG_NEUTRAL,                     "tn" },
+    { LANG_TSWANA,         SUBLANG_TSWANA_SOUTH_AFRICA,         "tn_ZA" },
     { LANG_TURKISH,        SUBLANG_NEUTRAL,                     "tr" },
     { LANG_TURKISH,        SUBLANG_TURKISH_TURKEY,              "tr_TR" },
-    { LANG_URDU,           SUBLANG_NEUTRAL,                     "ur" },
-    { LANG_URDU,           SUBLANG_URDU_PAKISTAN,               "ur_PK" },
-    { LANG_INDONESIAN,     SUBLANG_NEUTRAL,                     "id" },
-    { LANG_INDONESIAN,     SUBLANG_INDONESIAN_INDONESIA,        "id_ID" },
+    { LANG_UIGHUR,         SUBLANG_NEUTRAL,                     "ug" },
+    { LANG_UIGHUR,         SUBLANG_UIGHUR_PRC,                  "ug_CN" },
     { LANG_UKRAINIAN,      SUBLANG_NEUTRAL,                     "uk" },
     { LANG_UKRAINIAN,      SUBLANG_UKRAINIAN_UKRAINE,           "uk_UA" },
-    { LANG_BELARUSIAN,     SUBLANG_NEUTRAL,                     "be" },
-    { LANG_BELARUSIAN,     SUBLANG_BELARUSIAN_BELARUS,          "be_BY" },
-    { LANG_SLOVENIAN,      SUBLANG_NEUTRAL,                     "sl" },
-    { LANG_SLOVENIAN,      SUBLANG_SLOVENIAN_SLOVENIA,          "sl_SI" },
-    { LANG_ESTONIAN,       SUBLANG_NEUTRAL,                     "et" },
-    { LANG_ESTONIAN,       SUBLANG_ESTONIAN_ESTONIA,            "et_EE" },
-    { LANG_LATVIAN,        SUBLANG_NEUTRAL,                     "lv" },
-    { LANG_LATVIAN,        SUBLANG_LATVIAN_LATVIA,              "lv_LV" },
-    { LANG_LITHUANIAN,     SUBLANG_NEUTRAL,                     "lt" },
-    { LANG_LITHUANIAN,     SUBLANG_LITHUANIAN,                  "lt_LT" },
-    { LANG_PERSIAN,        SUBLANG_NEUTRAL,                     "fa" },
-    { LANG_PERSIAN,        SUBLANG_PERSIAN_IRAN,                "fa_IR" },
-    { LANG_ARMENIAN,       SUBLANG_NEUTRAL,                     "hy" },
-    { LANG_ARMENIAN,       SUBLANG_ARMENIAN_ARMENIA,            "hy_AM" },
-    { LANG_AZERI,          SUBLANG_NEUTRAL,                     "az" },
-    { LANG_AZERI,          SUBLANG_AZERI_LATIN,                 "az_AZ@latin" },
-    { LANG_AZERI,          SUBLANG_AZERI_CYRILLIC,              "az_AZ@cyrillic" },
-    { LANG_BASQUE,         SUBLANG_NEUTRAL,                     "eu" },
-    { LANG_BASQUE,         SUBLANG_BASQUE_BASQUE,               "eu_ES" },
-    { LANG_MACEDONIAN,     SUBLANG_NEUTRAL,                     "mk" },
-    { LANG_MACEDONIAN,     SUBLANG_MACEDONIAN_MACEDONIA,        "mk_MK" },
-    { LANG_AFRIKAANS,      SUBLANG_NEUTRAL,                     "af" },
-    { LANG_AFRIKAANS,      SUBLANG_AFRIKAANS_SOUTH_AFRICA,      "af_ZA" },
-    { LANG_GEORGIAN,       SUBLANG_NEUTRAL,                     "ka" },
-    { LANG_GEORGIAN,       SUBLANG_GEORGIAN_GEORGIA,            "ka_GE" },
-    { LANG_FAEROESE,       SUBLANG_NEUTRAL,                     "fo" },
-    { LANG_FAEROESE,       SUBLANG_FAEROESE_FAROE_ISLANDS,      "fo_FO" },
-    { LANG_HINDI,          SUBLANG_NEUTRAL,                     "hi" },
-    { LANG_HINDI,          SUBLANG_HINDI_INDIA,                 "hi_IN" },
-    { LANG_MALAY,          SUBLANG_NEUTRAL,                     "ms" },
-    { LANG_MALAY,          SUBLANG_MALAY_MALAYSIA,              "ms_MY" },
-    { LANG_MALAY,          SUBLANG_MALAY_BRUNEI_DARUSSALAM,     "ms_BN" },
-    { LANG_KAZAK,          SUBLANG_NEUTRAL,                     "kk" },
-    { LANG_KAZAK,          SUBLANG_KAZAK_KAZAKHSTAN,            "kk_KZ" },
-    { LANG_KYRGYZ,         SUBLANG_NEUTRAL,                     "ky" },
-    { LANG_KYRGYZ,         SUBLANG_KYRGYZ_KYRGYZSTAN,           "ky_KG" },
-    { LANG_SWAHILI,        SUBLANG_NEUTRAL,                     "sw" },
-    { LANG_SWAHILI,        SUBLANG_SWAHILI_KENYA,               "sw_KE" },
+    { LANG_URDU,           SUBLANG_NEUTRAL,                     "ur" },
+    { LANG_URDU,           SUBLANG_URDU_PAKISTAN,               "ur_PK" },
+    { LANG_URDU,           SUBLANG_URDU_INDIA,                  "ur_IN" },
     { LANG_UZBEK,          SUBLANG_NEUTRAL,                     "uz" },
     { LANG_UZBEK,          SUBLANG_UZBEK_LATIN,                 "uz_UZ@latin" },
     { LANG_UZBEK,          SUBLANG_UZBEK_CYRILLIC,              "uz_UZ@cyrillic" },
-    { LANG_TATAR,          SUBLANG_NEUTRAL,                     "tt" },
-    { LANG_TATAR,          SUBLANG_TATAR_RUSSIA,                "tt_TA" },
-    { LANG_PUNJABI,        SUBLANG_NEUTRAL,                     "pa" },
-    { LANG_PUNJABI,        SUBLANG_PUNJABI_INDIA,               "pa_IN" },
-    { LANG_GUJARATI,       SUBLANG_NEUTRAL,                     "gu" },
-    { LANG_GUJARATI,       SUBLANG_GUJARATI_INDIA,              "gu_IN" },
-    { LANG_ORIYA,          SUBLANG_NEUTRAL,                     "or" },
-    { LANG_ORIYA,          SUBLANG_ORIYA_INDIA,                 "or_IN" },
-    { LANG_TAMIL,          SUBLANG_NEUTRAL,                     "ta" },
-    { LANG_TAMIL,          SUBLANG_TAMIL_INDIA,                 "ta_IN" },
-    { LANG_TELUGU,         SUBLANG_NEUTRAL,                     "te" },
-    { LANG_TELUGU,         SUBLANG_TELUGU_INDIA,                "te_IN" },
-    { LANG_KANNADA,        SUBLANG_NEUTRAL,                     "kn" },
-    { LANG_KANNADA,        SUBLANG_KANNADA_INDIA,               "kn_IN" },
-    { LANG_MALAYALAM,      SUBLANG_NEUTRAL,                     "ml" },
-    { LANG_MALAYALAM,      SUBLANG_MALAYALAM_INDIA,             "ml_IN" },
-    { LANG_MARATHI,        SUBLANG_NEUTRAL,                     "mr" },
-    { LANG_MARATHI,        SUBLANG_MARATHI_INDIA,               "mr_IN" },
-    { LANG_SANSKRIT,       SUBLANG_NEUTRAL,                     "sa" },
-    { LANG_SANSKRIT,       SUBLANG_SANSKRIT_INDIA,              "sa_IN" },
-    { LANG_MONGOLIAN,      SUBLANG_NEUTRAL,                     "mn" },
-    { LANG_MONGOLIAN,      SUBLANG_MONGOLIAN_CYRILLIC_MONGOLIA, "mn_MN" },
+    { LANG_VIETNAMESE,     SUBLANG_NEUTRAL,                     "vi" },
+    { LANG_VIETNAMESE,     SUBLANG_VIETNAMESE_VIETNAM,          "vi_VN" },
     { LANG_WELSH,          SUBLANG_NEUTRAL,                     "cy" },
     { LANG_WELSH,          SUBLANG_WELSH_UNITED_KINGDOM,        "cy_GB" },
-    { LANG_GALICIAN,       SUBLANG_NEUTRAL,                     "gl" },
-    { LANG_GALICIAN,       SUBLANG_GALICIAN_GALICIAN,           "gl_ES" },
-    { LANG_KONKANI,        SUBLANG_NEUTRAL,                     "kok" },
-    { LANG_KONKANI,        SUBLANG_KONKANI_INDIA,               "kok_IN" },
-    { LANG_DIVEHI,         SUBLANG_NEUTRAL,                     "dv" },
-    { LANG_DIVEHI,         SUBLANG_DIVEHI_MALDIVES,             "dv_MV" },
-    { LANG_BRETON,         SUBLANG_NEUTRAL,                     "br" },
-    { LANG_BRETON,         SUBLANG_BRETON_FRANCE,               "br_FR" },
+    { LANG_WOLOF,          SUBLANG_NEUTRAL,                     "wo" },
+    { LANG_WOLOF,          SUBLANG_WOLOF_SENEGAL,               "wo_SN" },
+    { LANG_XHOSA,          SUBLANG_NEUTRAL,                     "xh" },
+    { LANG_XHOSA,          SUBLANG_XHOSA_SOUTH_AFRICA,          "xh_ZA" },
+    { LANG_YORUBA,         SUBLANG_NEUTRAL,                     "yo" },
+    { LANG_YORUBA,         SUBLANG_YORUBA_NIGERIA,              "yo_NG" },
+    { LANG_ZULU,           SUBLANG_NEUTRAL,                     "zu" },
+    { LANG_ZULU,           SUBLANG_ZULU_SOUTH_AFRICA,           "zu_ZA" },
 
 #ifdef LANG_ESPERANTO
     { LANG_ESPERANTO,      SUBLANG_DEFAULT,                     "eo" },
@@ -569,7 +645,7 @@ static po_file_t create_po_file(void)
     po_message_set_msgid( msg, "" );
     po_message_set_msgstr( msg,
                            "Project-Id-Version: Wine\n"
-                           "Report-Msgid-Bugs-To: http://bugs.winehq.org\n"
+                           "Report-Msgid-Bugs-To: https://bugs.winehq.org\n"
                            "POT-Creation-Date: N/A\n"
                            "PO-Revision-Date: N/A\n"
                            "Last-Translator: Automatically generated\n"
@@ -602,7 +678,7 @@ static const char *get_language_name( const language_t *lang )
     static char name[20];
     unsigned int i;
 
-    for (i = 0; i < sizeof(languages)/sizeof(languages[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(languages); i++)
         if (languages[i].id == lang->id && languages[i].sub == lang->sub)
             return languages[i].name;
 
@@ -1314,7 +1390,7 @@ static ver_value_t *translate_stringfileinfo( ver_value_t *val, language_t *lang
                 new_blk = new_ver_block();
                 *new_blk = *blk;
                 name = convert_msgid_ascii( blk->name, 0 );
-                for (i = 0; i < sizeof(block_name)/sizeof(block_name[0]); i++)
+                for (i = 0; i < ARRAY_SIZE(block_name); i++)
                 {
                     if (!strcasecmp( name, english_block_name[i] ))
                     {
@@ -1344,7 +1420,7 @@ static ver_value_t *translate_stringfileinfo( ver_value_t *val, language_t *lang
         val = val->next;
     }
 
-    for (i = 0; i < sizeof(block_name)/sizeof(block_name[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(block_name); i++)
         free( block_name[i] );
     return head;
 }
@@ -1494,10 +1570,10 @@ void add_translations( const char *po_dir )
         if ((p = strchr( buffer, '#' ))) *p = 0;
         for (tok = strtok( buffer, " \t\r\n" ); tok; tok = strtok( NULL, " \t\r\n" ))
         {
-            for (i = 0; i < sizeof(languages)/sizeof(languages[0]); i++)
+            for (i = 0; i < ARRAY_SIZE(languages); i++)
                 if (!strcmp( tok, languages[i].name )) break;
 
-            if (i == sizeof(languages)/sizeof(languages[0]))
+            if (i == ARRAY_SIZE(languages))
                 error( "unknown language '%s'\n", tok );
 
             name = strmake( "%s/%s.mo", po_dir, tok );
